/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)PjTabNotebook.java  0.00 05-Jan-96

    An implementation of PjNbSpec using TabNotebook.

    Authors:

        jts      Ted Skolnick
        lning    Lindee Ning
        rphall   Rick Hall
        jlee     James Lee

    Version Ident:

        $Header: /PjJavaClient/src/pj/awt/PjTabNotebook.java 6     2/02/96 11:46a Rphall $

    History:

        0.00 05-Jan-96  rphall      Initial Creation
             29-Jan-96  jlee        Added WeatherSection
             12-Feb-96  ln          Added ticker
             13-Feb-96  jts         Added DogEar
             14-Feb-96  jlee        Added SportsSection
             23-Feb-96  Ted S.      Made this implement PaperView so you
                                    can step through the paper saying NextView().
              4-Mar-96  jlee        Changed class name from "PjCheckboxNb" to "PjTabNotebook"
                                    Used "TabNotebook" in place for "CheckboxNb"
              5-Mar-96  jlee        Added status bar and current time bar.
              9-Mar-96  lning       Added the function to handle the menu selection.
             21-Mar-96  jlee        Added download progress bar.
             25-Mar-96  jlee        Used Panel3D(int) instead Panel3d(int,int,int,int).
             26-Mar-96  Ted S.      Deleted a stray semicolon that prevented compiling.
                                    And added a missing one.
             27-Mar-96  jlee        Used Panel3D to contain ticker and dogear.
---------------------------------------------------------------------------*/

package pj.awt;


import pj.awt.DogEar;
import pj.awt.HeadlineSection;
import pj.awt.MarketSection;

import pj.awt.Page;
import pj.awt.Panel3D;
import pj.awt.PaperView;
import pj.awt.PjImages;
import pj.awt.PjMenuItem;
import pj.awt.PjNbSpec;
import pj.awt.PjTicker;
import pj.awt.Section;
import pj.awt.SportsSection;
import pj.awt.StatusBar;
import pj.awt.TabNotebook;
import pj.awt.WeatherSection;
import pj.io.Paper;
import pj.util.LocalDateTime;

import collections.Assertable;
import collections.ImplementationError;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Panel;
import java.awt.Event;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.MenuItem;
import java.awt.Menu;
import java.awt.MenuComponent;
import java.awt.Component;
import java.util.Date;

/**
 * An implementation of PjNbSpec using TabNotebook.
 *
 * @see     pj.awt.TabNotebook
 * @version 0.00 05-Jan-96
 * @author  rphall
*/
public class PjTabNotebook extends TabNotebook implements Assertable
    {

    // --- Class varibales

    private static  StatusBar   sbMessage = null;

    // --- Instance variables

    private DogEar   dogear;
    private Paper    paper;
    private PjTicker ticker;

    // --- Public constructors

    /**
     * Create a Personal Journal notebook.
    */
        public PjTabNotebook(Paper p, PjAds ads, PjImages images)
        {
        assert(p != null);
        assert(ads != null);
        assert(images != null);

        paper = p;
        PjNbSpec nbs = new PjNbSpec();

        appendPage( new HeadlineSection(
                nbs.pagespecs()[nbs.idxFPHdlines],paper) );

        appendPage( new HeadlineSection(
                nbs.pagespecs()[nbs.idxTJIHdlines],paper) );

        appendPage( new MarketSection(
                nbs.pagespecs()[nbs.idxMrk],paper) );

        appendPage( new PortView(
                nbs.pagespecs()[nbs.idxPort],
                ads.adspecs()[ads.idxPortAd],paper) );

        appendPage( new SportsSection(
                nbs.pagespecs()[nbs.idxSpHilite],
                ads.adspecs()[ads.idxSpAd],paper) );

        appendPage( new WeatherSection(
                nbs.pagespecs()[nbs.idxWth],
                ads.adspecs()[ads.idxWthAd],paper,
                images.imagespecs()[images.idxWthMap]) );


        // Top panel with Ticker, DogEar
        Panel pnlTop = new Panel();
            pnlTop.setLayout( new BorderLayout(1, 1) );
            Panel3D pnl3dTicker = new Panel3D();
                Panel pnl = new Panel();
                    GridBagLayout gridbag   = new GridBagLayout();
                    GridBagConstraints gbc  = new GridBagConstraints();
                    gbc.insets = new Insets( 1, 3, 0, 2 );
                    pnl.setLayout( gridbag );
        
                    ticker = new PjTicker( p.idPortQuote, p );
                    ticker.startTicker();
                    gbc.weightx = 1.0;
                    gbc.fill = GridBagConstraints.HORIZONTAL;
                    gbc.gridwidth = GridBagConstraints.RELATIVE;
                    gridbag.setConstraints(ticker,gbc);
                    pnl.add(ticker);
                pnl3dTicker.add( pnl );
            pnlTop.add("Center", pnl3dTicker);

            Panel3D pnl3dDogear = new Panel3D();
                dogear = new DogEar( images, this );
                pnl3dDogear.add( dogear );
            pnlTop.add("East", pnl3dDogear);

        add( pnlTop );

        // Create status bars
        Panel3D pnl3dStatusBar1 = new Panel3D( 2 );
        Panel3D pnl3dStatusBar2 = new Panel3D( 2 );
            // Download progress bar setting
            sbMessage =  new StatusBar();
            sbMessage.setMessage( "Downloading news from the Wall Street Jorunal...");
            paper.addObserver( sbMessage );

            // Add status bars to status panels
            pnl3dStatusBar1.add( sbMessage );
            pnl3dStatusBar2.add( new LocalDateTime() );

        add( pnl3dStatusBar1 );
        add( pnl3dStatusBar2 );

        System.out.println("Debug-PjTabNotebook:constructed");
        } // PjTabNotebook

    // --- Public operations

    /**
     * Raise an exception if predicate is false.
     * @see collections.Assertable
    */
    public void assert(boolean predicate) throws ImplementationError
        {
        ImplementationError.assert(this, predicate);
        }

    /**
     * Returns the StatusBar object
     * @return The StatusBar used in the applet
    */
    public static StatusBar getStatusBar()
        {
        return sbMessage;
        }

    /**
     * Turning to the page selected from the menubar
    */
    public boolean action(Event evt, Object arg)
        {
        if(evt.target instanceof PjMenuItem)
            {
            boolean     found = false;
            String      mainmenuData = new String();
            String      pagenamestr = ((PjMenuItem)(evt.target)).getLabel(); //String name of the page in the section
            String      pagename = ((PjMenuItem)(evt.target)).getItemData(); //Internal name of the page
            String      menustr =((Menu)(((MenuComponent)(evt.target)).getParent())).getLabel(); //Section name (String)
            PjNbSpec    nbs = new PjNbSpec();

            for(int i = 0; i < nbs.mainmenustring.length; i++)
                {
                if(menustr == nbs.mainmenustring[i])
                    {
                    found = true;
                    mainmenuData = nbs.mainmenuname[i];//Section name(Internal name)
                    break;
                    }
                }//for

            if (!found)
                mainmenuData = pagename;

            if(found) //submenuitem
                {
                if(pagename == mainmenuData) //first page of the section
                    {
                    page(pagename);
                    return true;
                    }
                else
                    {
                    page(mainmenuData);
                    for(int i = 0; i < nbs.menuname.length; i++)
                        {
                        if(pagename == nbs.menuname[i])
                            {
                            pagenamestr = nbs.menutosection[i];
                            break;
                            }
                        }//for

                    int idx = 0;
                    for(int i = 0; i < nbs.mainmenuname.length; i++)
                        {
                        if(mainmenuData == nbs.mainmenuname[i])
                            {
                            idx = i;
                            break;
                            }
                        }//for

                    switch ( idx )
                    {
                    case 0:
                        (stack.getComponent(0)).action(evt, pagenamestr);
                        break;
                    case 1:
                        (stack.getComponent(1)).action(evt, pagenamestr);
                        break;
                    case 2:
                        (stack.getComponent(2)).action(evt, pagenamestr);
                        break;
                    case 3:
                        (stack.getComponent(3)).action(evt, pagenamestr);
                        break;
                    case 4:
                        (stack.getComponent(4)).action(evt, pagenamestr);
                        break;
                    case 5:
                        (stack.getComponent(5)).action(evt, pagenamestr);
                        break;

                    default:
                        break;
                    }//switch

                    return true;
                    }//if
                }//if
            else
                {
                page(pagename);
                return true;
                }
            }//if

        return false;
        }




    } // PjTabNotebook
