/*---------------------------------------------------------------------------

	Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    StringTabSpec.java

        A string-based specification for a component of a TabBar.


	Authors:

        rphall          Rick Hall


	Version Ident:

        $Header: /PjJavaClient/src/pj/awt/StringTabSpec.java 2     2/08/96 8:57a Rphall $


	History:

        10-dec-1995 rphalll     Initial Creation


---------------------------------------------------------------------------*/

package pj.awt;

import pj.awt.TabSpec;

import collections.Assertable;
import collections.ImplementationCheckable;
import collections.ImplementationError;
import java.lang.Object;
import java.lang.String;

/**
 * A string-based specification for a component of a TabBar.
 *
 * @see TabSpec
 * @version 0.00 03-Jan-96
 * @author Rick Hall
*/
public class StringTabSpec extends TabSpec
    implements ImplementationCheckable
    {

    // --- Public constructors

    /**
     * @param tabName The internal id of a tab within a TabBar.
     * @param pageName The pageName of the divider to which this tab refers.
     * @param tabString A string with which this tab will be labelled.
     * @see checkImplementation
    */
    public StringTabSpec(
            String tabName,
            String pageName,
            String tabString)
        { 
        super(tabName,pageName,tabString); 
        }

    // --- Public operations

    /**
     * Checks the internal validity of a StringTabSpec by making
     * sure that StringTabSpec.objVisSpec is an instance of String.
     * Invokes super to check that no public attribute is null.
     * @exception ImplementationError Thrown if objVisSpec is not
     * a String.
    */
    public void checkImplementation() throws ImplementationError
        {
        super.checkImplementation();
        assert(objVisSpec instanceof String);
        }

    } // TabSpec
