##############################################################################
#
#  Copyright (c) Dow Jones and Company. 1994,1995. All Rights Reserved.
#
#  Io.mak
#
#       Makefile for testing observable sections in an observable paper.
#
#  Authors:
#
#      rphall   Rick Hall
#
#  Version Ident:
#
#      $Header: /PjJavaClient/src/pj/io/Io.mak 3     1/30/96 6:18p Rphall $
#
#  History:
#
#      12/26/95 rphall  initial creation
#
##############################################################################

# REQUIRED: define where master makefiles can be found.
# (Make definition conditional so that it can be overridden on a command line.)
!IF "$(JAVAMAKEDIR)" == ""
JAVAMAKEDIR=.\..\..
!ENDIF
!IF "$(CFG)" == ""
CFG=Java Debug
!ENDIF
!IF "$(JAVACLASSDIR)" == ""
JAVACLASSDIR=.\classes
!ENDIF
!IF "$(JAVADOCDIR)" == ""
JAVADOCDIR=.\apidocs
!ENDIF

# Include default macro definitions and generalized rules
!INCLUDE $(JAVAMAKEDIR)\defaults.mak

# List your classes here, one per line, with a backslash for continuation
CLASSES=CUP$$actions.class          \
        DownloadProgressNotification.class  \
        IncompatiblePaperStoryException.class  \
        MalformedPaperStoryException.class     \
        Paper.class                 \
        PaperSection.class          \
        PaperStory.class            \
        ParserThread.class          \
        StoryContainer.class        \
        StoryContainerNotification.class \
        StatefulStoryContainer.class \
        StoryScanner.class          \
        StoryParser.class           \
        StoryParserSymbols.class

# List the classes you want documented, one per line, with backslashes
APIDOCS=Packages-pj.io.html \
        pj.io.DownloadProgressNotification.html  \
        pj.io.IncompatiblePaperStoryException.html  \
        pj.io.MalformedPaperStoryException.html     \
        pj.io.Paper.html                    \
        pj.io.PaperSection.html             \
        pj.io.PaperStory.html               \
        pj.io.ParserThread.html              \
        pj.io.StoryContainer.html           \
        pj.io.StatefulStoryContainer.html   \
        pj.io.StoryParser.html              \
        pj.io.StoryScanner.html

# The default target for the build. (The first target is the project default.)
binary : $(JAVACLASSDIR) $(CLASSES)
        for %f in ($(CLASSES)) do copy %f $(JAVACLASSDIR)

documentation : $(JAVADOCDIR) $(JAVADOCDIR)/images $(APIDOCS)
        for %f in ($(APIDOCS)) do copy %f $(JAVADOCDIR)

all : binary documentation

# Include other targets
!INCLUDE $(JAVAMAKEDIR)\dtargets.mak

# List dependencies here (particularly private class dependencies)

CUP$$actions.class : StoryParser.java

DownloadProgressNotification.class      : DownloadProgressNotification.java

Paper.class :       Paper.java \
                    IncompatiblePaperStoryException.class \
                    MalformedPaperStoryException.class \
                    PaperStory.class \
                    StoryContainer.class \
                    PaperSection.class

PaperSection.class : PaperSection.java \
                    DownloadProgressNotification.class \
                    IncompatiblePaperStoryException.class \
                    MalformedPaperStoryException.class \
                    PaperStory.class \
                    StoryContainer.class \
                    StoryContainerNotification.class

PaperStory.class :  PaperStory.java \
                    MalformedPaperStoryException.class

ParserThread.class : ParserThread.java \
                    Paper.class \
                    PaperSection.class \
                    StoryContainerNotification.class \
                    StoryParser.class

StoryContainer.class : StoryContainer.java \
                    IncompatiblePaperStoryException.class \
                    MalformedPaperStoryException.class

StoryContainerNotification.class : StoryContainerNotification.java

StoryParser.class : StoryParser.java \
                    CUP$$actions.class \
                    StoryParserSymbols.class \
                    StoryScanner.class  \
                    StoryContainer.class \
                    PaperStory.class

StoryScanner.class : StoryScanner.java \
                    StoryParserSymbols.class \
                    PaperStory.class

pj.io.IncompatiblePaperStoryException.html : IncompatiblePaperStoryException.html

pj.io.IOTest.html                          : IOTest.html

pj.io.MalformedPaperStoryException.html    : MalformedPaperStoryException.html

pj.io.Paper.html                           : Paper.html

pj.io.PaperSection.html                    : PaperSection.html

pj.io.PaperStory.html                      : PaperStory.html

pj.io.ParserThread.html                     : ParserThread.html

pj.io.StatefulStoryContainer.html          : StatefulStoryContainer.html

pj.io.StoryContainer.html                  : StoryContainer.html

pj.io.StoryParser.html                     : StoryParser.html

pj.io.StoryScanner.html                    : StoryScanner.html
