##############################################################################
#
#  Copyright (c) 1996 by Dow Jones and Company, Inc.
#
#  pj.mak
#
#       Makefile for pj package
#
#  Authors:
#
#      rphall   Rick Hall
#
#  Version Ident:
#
#      $Header: /PjJavaClient/src/pj/pj.mak 9     2/05/96 1:30p Rphall $
#
#  History:
#      17-Jan-96    rphall  Modifications to make more machine independent
#
#      11-Jan-96    rphall  Initial creation
#
##############################################################################

!IF "$(SRCPATH)" == ""
SRCPATH=$(MAKEDIR)\..
!ENDIF

!IF "$(OUTCLASSPATH)" == ""
OUTCLASSPATH=$(MAKEDIR)\..\..\CLASSES
PJCLASSDIR=$(OUTCLASSPATH)\PJ
!ENDIF

!MESSAGE
!MESSAGE Source path = $(SRCPATH)
!MESSAGE Output path = $(OUTCLASSPATH)
!MESSAGE Classpath = $(CLASSPATH)

!IF "$(OUTCLASSPATH);$(SRCPATH)" != "$(CLASSPATH)"
!ERROR Set CLASSPATH=$(OUTCLASSPATH);$(SRCPATH) before running this makefile
!ENDIF

!IF "$(OUTDOCPATH)" == ""
OUTDOCPATH=$(MAKEDIR)\..\..\APIDOCS
!ENDIF

!IF "$(TARGET)" == ""
TARGET=binary
!ENDIF

# REQUIRED: define where master makefiles can be found.
# (Make definition conditional so that it can be overridden on a command line.)
!IF "$(JAVAMAKEDIR)" == ""
JAVAMAKEDIR=.\..
!ENDIF
!IF "$(CFG)" == ""
CFG=Java Debug
!ENDIF

# Include default macro definitions and generalized rules
!INCLUDE $(JAVAMAKEDIR)\defaults.mak

# Subproject output directories

APPLET_OUT=$(PJCLASSDIR)\applet
AWT_OUT=$(PJCLASSDIR)\awt
IO_OUT=$(PJCLASSDIR)\io
LANG_OUT=$(PJCLASSDIR)\lang
NET_OUT=$(PJCLASSDIR)\net
UTIL_OUT=$(PJCLASSDIR)\util

# Subproject source directories

APPLET_SRC=$(SRCPATH)\pj\applet
AWT_SRC=$(SRCPATH)\pj\awt
IO_SRC=$(SRCPATH)\pj\io
LANG_SRC=$(SRCPATH)\pj\lang
NET_SRC=$(SRCPATH)\pj\net
UTIL_SRC=$(SRCPATH)\pj\util

# The default target for the build. (The first target is the project default.)
binary : $(PJCLASSDIR) $(APPLET_OUT) $(AWT_OUT) $(IO_OUT) $(NET_OUT) $(UTIL_OUT) \
         APPLET_CLASSES AWT_CLASSES IO_CLASSES NET_CLASSES UTIL_CLASSES

documentation : $(OUTDOCPATH)
    cd $(APPLET_SRC)
    nmake /f Applet.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(AWT_SRC)
    nmake /f Awt.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(IO_SRC)
    nmake /f Io.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(LANG_SRC)
    nmake /f Lang.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(NET_SRC)
    nmake /f Net.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(UTIL_SRC)
    nmake /f Util.mak $(TARGET) documentation JAVADOCDIR=$(OUTDOCPATH)
    cd $(SRCPATH)\pj
    copy Packages-PjJava.html $(OUTDOCPATH)

all : binary documentation

cleanup :
    cd $(APPLET_SRC)
    nmake /f Applet.mak clean
    cd $(AWT_SRC)
    nmake /f Awt.mak clean
    cd $(IO_SRC)
    nmake /f Io.mak clean
    cd $(LANG_SRC)
    nmake /f Lang.mak clean
    cd $(NET_SRC)
    nmake /f Net.mak clean
    cd $(UTIL_SRC)
    nmake /f Util.mak clean
    cd $(SRCPATH)
    attrib -r mssccprj.scc
    del mssccprj.scc
    cd $(SRCPATH)\pj
    attrib -r mssccprj.scc
    del mssccprj.scc

allAndClean : all cleanup

$(PJCLASSDIR) : 
    if not exist $(PJCLASSDIR)/nul mkdir $(PJCLASSDIR)

$(OUTDOCPATH) :
    if not exist $(OUTDOCPATH)/nul mkdir $(OUTDOCPATH)

$(APPLET_OUT) :
    if not exist $(APPLET)/nul mkdir $(APPLET_OUT)

$(AWT_OUT) :
    if not exist $(AWT)/nul mkdir $(AWT_OUT)

$(IO_OUT) :
    if not exist $(IO)/nul mkdir $(IO_OUT)

$(LANG_OUT) :
    if not exist $(LANG)/nul mkdir $(LANG_OUT)

$(NET_OUT) :
    if not exist $(NET)/nul mkdir $(NET_OUT)

$(UTIL_OUT) :
    if not exist $(UTIL)/nul mkdir $(UTIL_OUT)

# Subproject targets

APPLET_CLASSES: NET_CLASSES IO_CLASSES AWT_CLASSES UTIL_CLASSES
    cd $(APPLET_SRC)
    nmake /f Applet.mak $(TARGET) JAVACLASSDIR=$(APPLET_OUT)
    cd $(PJCLASSDIR)

AWT_CLASSES: IO_CLASSES UTIL_CLASSES
    cd $(AWT_SRC)
    nmake /f Awt.mak $(TARGET) JAVACLASSDIR=$(AWT_OUT)
    cd $(PJCLASSDIR)

IO_CLASSES: UTIL_CLASSES
    cd $(IO_SRC)
    nmake /f Io.mak $(TARGET) JAVACLASSDIR=$(IO_OUT)
    cd $(PJCLASSDIR)

LANG_CLASSES:
    cd $(LANG_SRC)
    nmake /f Lang.mak $(TARGET) JAVACLASSDIR=$(LANG_OUT)
    cd $(PJCLASSDIR)

NET_CLASSES: LANG_CLASSES
    cd $(NET_SRC)
    nmake /f Net.mak $(TARGET) JAVACLASSDIR=$(NET_OUT)
    cd $(PJCLASSDIR)

UTIL_CLASSES:
    cd $(UTIL_SRC)
    nmake /f Util.mak $(TARGET) JAVACLASSDIR=$(UTIL_OUT)
    cd $(PJCLASSDIR)

# Include other targets
!INCLUDE $(JAVAMAKEDIR)\dtargets.mak

# List dependencies here (particularly private class dependencies)
